"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var input_event_1 = require("./input-event");
var InputEventMapper = /** @class */ (function () {
    function InputEventMapper(longSize, getLong, setLong) {
        this.longSize = longSize;
        this.getLong = getLong;
        this.setLong = setLong;
    }
    InputEventMapper.for = function (is64Bit) {
        return is64Bit ? this.for64bit() : this.for32bit();
    };
    InputEventMapper.for64bit = function () {
        return new InputEventMapper(BigInt64Array.BYTES_PER_ELEMENT, function (offset, data) { return data.getBigInt64(offset, true); }, function (offset, value, data) { return data.setBigInt64(offset, value, true); });
    };
    InputEventMapper.for32bit = function () {
        return new InputEventMapper(Int32Array.BYTES_PER_ELEMENT, function (offset, data) { return BigInt(data.getInt32(offset, true)); }, function (offset, value, data) { return data.setInt32(offset, Number(value), true); });
    };
    Object.defineProperty(InputEventMapper.prototype, "eventSize", {
        get: function () {
            return 2 * this.longSize + 2 * Int16Array.BYTES_PER_ELEMENT + Int32Array.BYTES_PER_ELEMENT;
        },
        enumerable: true,
        configurable: true
    });
    InputEventMapper.prototype.fromBuffer = function (buffer) {
        var data = new DataView(buffer);
        var offset = 0;
        var sec = this.getLong(offset, data);
        offset += this.longSize;
        var usec = this.getLong(offset, data);
        offset += this.longSize;
        var type = data.getUint16(offset, true);
        offset += Int16Array.BYTES_PER_ELEMENT;
        var code = data.getUint16(offset, true);
        offset += Int16Array.BYTES_PER_ELEMENT;
        var value = data.getUint32(offset, true);
        offset += Int32Array.BYTES_PER_ELEMENT;
        return new input_event_1.InputEvent(sec, usec, type, code, value);
    };
    InputEventMapper.prototype.toBuffer = function (event, buffer) {
        if (buffer === undefined) {
            buffer = new ArrayBuffer(this.eventSize);
        }
        else if (buffer.byteLength < this.eventSize) {
            throw new RangeError("Expected buffer of size " + this.eventSize + ", got " + buffer.byteLength);
        }
        var data = new DataView(buffer);
        var offset = 0;
        this.setLong(offset, event.sec, data);
        offset += this.longSize;
        this.setLong(offset, event.usec, data);
        offset += this.longSize;
        data.setUint16(offset, event.type, true);
        offset += Int16Array.BYTES_PER_ELEMENT;
        data.setUint16(offset, event.code, true);
        offset += Int16Array.BYTES_PER_ELEMENT;
        data.setUint32(offset, event.value, true);
        offset += Int32Array.BYTES_PER_ELEMENT;
        return buffer;
    };
    InputEventMapper.prototype.fromIterable = function (iterable) {
        var buffer, offset, iterable_1, iterable_1_1, chunk, _a, _b, byte, e_1_1, e_2_1;
        var e_2, _c, e_1, _d;
        return tslib_1.__generator(this, function (_e) {
            switch (_e.label) {
                case 0:
                    buffer = new Uint8Array(this.eventSize);
                    offset = 0;
                    _e.label = 1;
                case 1:
                    _e.trys.push([1, 12, 13, 14]);
                    iterable_1 = tslib_1.__values(iterable), iterable_1_1 = iterable_1.next();
                    _e.label = 2;
                case 2:
                    if (!!iterable_1_1.done) return [3 /*break*/, 11];
                    chunk = iterable_1_1.value;
                    _e.label = 3;
                case 3:
                    _e.trys.push([3, 8, 9, 10]);
                    _a = (e_1 = void 0, tslib_1.__values(new Uint8Array(chunk))), _b = _a.next();
                    _e.label = 4;
                case 4:
                    if (!!_b.done) return [3 /*break*/, 7];
                    byte = _b.value;
                    buffer[offset++] = byte;
                    if (offset < buffer.length) {
                        return [3 /*break*/, 6];
                    }
                    return [4 /*yield*/, this.fromBuffer(buffer.buffer)];
                case 5:
                    _e.sent();
                    buffer = new Uint8Array(this.eventSize);
                    offset = 0;
                    _e.label = 6;
                case 6:
                    _b = _a.next();
                    return [3 /*break*/, 4];
                case 7: return [3 /*break*/, 10];
                case 8:
                    e_1_1 = _e.sent();
                    e_1 = { error: e_1_1 };
                    return [3 /*break*/, 10];
                case 9:
                    try {
                        if (_b && !_b.done && (_d = _a.return)) _d.call(_a);
                    }
                    finally { if (e_1) throw e_1.error; }
                    return [7 /*endfinally*/];
                case 10:
                    iterable_1_1 = iterable_1.next();
                    return [3 /*break*/, 2];
                case 11: return [3 /*break*/, 14];
                case 12:
                    e_2_1 = _e.sent();
                    e_2 = { error: e_2_1 };
                    return [3 /*break*/, 14];
                case 13:
                    try {
                        if (iterable_1_1 && !iterable_1_1.done && (_c = iterable_1.return)) _c.call(iterable_1);
                    }
                    finally { if (e_2) throw e_2.error; }
                    return [7 /*endfinally*/];
                case 14: return [2 /*return*/];
            }
        });
    };
    InputEventMapper.prototype.toIterable = function (iterable) {
        var iterable_2, iterable_2_1, event, e_3_1;
        var e_3, _a;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _b.trys.push([0, 5, 6, 7]);
                    iterable_2 = tslib_1.__values(iterable), iterable_2_1 = iterable_2.next();
                    _b.label = 1;
                case 1:
                    if (!!iterable_2_1.done) return [3 /*break*/, 4];
                    event = iterable_2_1.value;
                    return [4 /*yield*/, this.toBuffer(event)];
                case 2:
                    _b.sent();
                    _b.label = 3;
                case 3:
                    iterable_2_1 = iterable_2.next();
                    return [3 /*break*/, 1];
                case 4: return [3 /*break*/, 7];
                case 5:
                    e_3_1 = _b.sent();
                    e_3 = { error: e_3_1 };
                    return [3 /*break*/, 7];
                case 6:
                    try {
                        if (iterable_2_1 && !iterable_2_1.done && (_a = iterable_2.return)) _a.call(iterable_2);
                    }
                    finally { if (e_3) throw e_3.error; }
                    return [7 /*endfinally*/];
                case 7: return [2 /*return*/];
            }
        });
    };
    InputEventMapper.prototype.fromAsyncIterable = function (iterable) {
        return tslib_1.__asyncGenerator(this, arguments, function fromAsyncIterable_1() {
            var buffer, offset, iterable_3, iterable_3_1, chunk, _a, _b, byte, e_4_1, e_5_1;
            var e_4, _c;
            var e_5, _d;
            return tslib_1.__generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        buffer = new Uint8Array(this.eventSize);
                        offset = 0;
                        _e.label = 1;
                    case 1:
                        _e.trys.push([1, 14, 15, 20]);
                        iterable_3 = tslib_1.__asyncValues(iterable);
                        _e.label = 2;
                    case 2: return [4 /*yield*/, tslib_1.__await(iterable_3.next())];
                    case 3:
                        if (!(iterable_3_1 = _e.sent(), !iterable_3_1.done)) return [3 /*break*/, 13];
                        chunk = iterable_3_1.value;
                        _e.label = 4;
                    case 4:
                        _e.trys.push([4, 10, 11, 12]);
                        _a = (e_4 = void 0, tslib_1.__values(new Uint8Array(chunk))), _b = _a.next();
                        _e.label = 5;
                    case 5:
                        if (!!_b.done) return [3 /*break*/, 9];
                        byte = _b.value;
                        buffer[offset++] = byte;
                        if (offset < buffer.length) {
                            return [3 /*break*/, 8];
                        }
                        return [4 /*yield*/, tslib_1.__await(this.fromBuffer(buffer.buffer))];
                    case 6: return [4 /*yield*/, _e.sent()];
                    case 7:
                        _e.sent();
                        buffer = new Uint8Array(this.eventSize);
                        offset = 0;
                        _e.label = 8;
                    case 8:
                        _b = _a.next();
                        return [3 /*break*/, 5];
                    case 9: return [3 /*break*/, 12];
                    case 10:
                        e_4_1 = _e.sent();
                        e_4 = { error: e_4_1 };
                        return [3 /*break*/, 12];
                    case 11:
                        try {
                            if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                        }
                        finally { if (e_4) throw e_4.error; }
                        return [7 /*endfinally*/];
                    case 12: return [3 /*break*/, 2];
                    case 13: return [3 /*break*/, 20];
                    case 14:
                        e_5_1 = _e.sent();
                        e_5 = { error: e_5_1 };
                        return [3 /*break*/, 20];
                    case 15:
                        _e.trys.push([15, , 18, 19]);
                        if (!(iterable_3_1 && !iterable_3_1.done && (_d = iterable_3.return))) return [3 /*break*/, 17];
                        return [4 /*yield*/, tslib_1.__await(_d.call(iterable_3))];
                    case 16:
                        _e.sent();
                        _e.label = 17;
                    case 17: return [3 /*break*/, 19];
                    case 18:
                        if (e_5) throw e_5.error;
                        return [7 /*endfinally*/];
                    case 19: return [7 /*endfinally*/];
                    case 20: return [2 /*return*/];
                }
            });
        });
    };
    InputEventMapper.prototype.toAsyncIterable = function (iterable) {
        return tslib_1.__asyncGenerator(this, arguments, function toAsyncIterable_1() {
            var iterable_4, iterable_4_1, event, e_6_1;
            var e_6, _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _b.trys.push([0, 7, 8, 13]);
                        iterable_4 = tslib_1.__asyncValues(iterable);
                        _b.label = 1;
                    case 1: return [4 /*yield*/, tslib_1.__await(iterable_4.next())];
                    case 2:
                        if (!(iterable_4_1 = _b.sent(), !iterable_4_1.done)) return [3 /*break*/, 6];
                        event = iterable_4_1.value;
                        return [4 /*yield*/, tslib_1.__await(this.toBuffer(event))];
                    case 3: return [4 /*yield*/, _b.sent()];
                    case 4:
                        _b.sent();
                        _b.label = 5;
                    case 5: return [3 /*break*/, 1];
                    case 6: return [3 /*break*/, 13];
                    case 7:
                        e_6_1 = _b.sent();
                        e_6 = { error: e_6_1 };
                        return [3 /*break*/, 13];
                    case 8:
                        _b.trys.push([8, , 11, 12]);
                        if (!(iterable_4_1 && !iterable_4_1.done && (_a = iterable_4.return))) return [3 /*break*/, 10];
                        return [4 /*yield*/, tslib_1.__await(_a.call(iterable_4))];
                    case 9:
                        _b.sent();
                        _b.label = 10;
                    case 10: return [3 /*break*/, 12];
                    case 11:
                        if (e_6) throw e_6.error;
                        return [7 /*endfinally*/];
                    case 12: return [7 /*endfinally*/];
                    case 13: return [2 /*return*/];
                }
            });
        });
    };
    return InputEventMapper;
}());
exports.InputEventMapper = InputEventMapper;
